header{* Syntax and basic operations/properties *}

theory MonadBase 
imports MonadAxms
begin

text {* eta translation *}

syntax 
 "\<eta>" :: "'a \<Rightarrow> 'a M" 
translations 
 "\<eta>" == "eta"

text {* Declares monad syntax *}

constdefs 
 bind' :: "'a M \<Rightarrow> 'b M \<Rightarrow> 'b M"          (infixl ">>" 5)
"bind' p q \<equiv> bind p (\<lambda>x. q)"

text{* This sets up a Haskell-style "{...}" syntax. *}

nonterminals
  monseq

syntax
  "_monseq"  :: "monseq=> 'a M"                   ("{(_)}"    [5] 5)
  "_mongen"  :: "[idts, 'a M, monseq]=> monseq"   ("_<-(_);//_"  [10, 6, 5] 5)
  "_monexp"  :: "['a M, monseq]=> monseq"         ("_;//_"       [6, 5] 5)
  "_monexp0" :: "['a M] => monseq"                ("(_)"         5)

translations
  "_monseq(_mongen x a r)" => "a >>= (% x.(_monseq r))"
  "_monseq(_monexp a r)"   => "a >> (_monseq r)"
  "_monseq(_monexp0 a)"    => "a"
  (* Retranslation of bind/seq into sequence notation *)
  "_monseq(_mongen x p q)"    <= "p >>= (%x. q)"
  "_monseq(_monexp p q)"      <= "p >> q"
  (* Normalization macros *)
  "_monseq(_monexp p q)"      <= "_monseq (_monexp p (_monseq q))"
  "_monseq(_mongen x p q)"    <= "_monseq (_mongen x p (_monseq q))"


section{* Derived monad laws. *}

text{* Setting up the simplifier. *}

declare mon_lunit [simp]
        mon_runit [simp]
        mon_assoc [simp]
         
text{* More flavours of unit *}

theorem mon_lunit2 [simp]: "({\<eta> x; p}) = p"
  by (unfold bind'_def, simp)

theorem mon_runite [simp]: "({x<- p; \<eta> x}) = p"
  by simp


theorem mon_runit2 [simp]: "({p; \<eta> ()}) = p" 
  by (unfold bind'_def, simp)


text{* 
  We need associativity in all different flavours with @term{bind'} 
  instead of @term{bind'}.
  We also need a better way of naming these :-)
  All of these lemmas should enable us to always message monadic sequences
  into a right-associative normal form like.
  *}

lemma mon_assoc2 [simp]: "({({x<- s; t x}); u}) = ({x<- s; t x; u})"
  by (unfold bind'_def, simp)

lemma mon_assoc4 [simp]: "({x<-({s; t}); u x}) = ({s; x<- t; u x})"
  by (unfold bind'_def, simp)

lemma mon_assoc5 [simp]: "({({s; t}); u})= ({s; t; u})"
  by (unfold bind'_def, simp)


section{* Map and join *}

constdefs
 mapM  :: "('a \<Rightarrow> 'b) \<Rightarrow> 'a M \<Rightarrow> 'b M"
"mapM f m \<equiv> ({a<-m; \<eta> (f a)})"

 joinM :: " ('a M) M \<Rightarrow> 'a M "
"joinM m \<equiv> ({a<-m; a})"


lemma mapM_eta[simp] : " (mapM f) \<circ> \<eta> = \<eta> \<circ> f " 
by (rule ext, simp add: mapM_def)


lemma mapM_comp[simp] : " (mapM f) \<circ> (mapM g) = mapM (f \<circ> g) "
by (rule ext, simp add: mapM_def)


lemma mapM_id[simp] : "mapM id = id"
by (rule ext, simp add: mapM_def)

lemma joinM_eta : " joinM \<circ> \<eta> = id "
by (rule ext, simp add: joinM_def)


lemma mapM_joinM[simp] : " joinM (mapM f m) = (m >>= f) "
by(simp add: joinM_def mapM_def)

lemma mapM_joinM_comp[simp] : " joinM \<circ> (mapM \<eta>) = id "
by(rule ext, simp)

lemma mapM_joinM_comp2[simp] : " joinM \<circ> mapM (mapM f) = (mapM f) \<circ> joinM "
by (rule ext, simp add: mapM_def joinM_def)

lemma joinM_mapM_joinM[simp] : "joinM \<circ> mapM joinM = joinM \<circ> joinM"
by (rule ext, simp add: mapM_def joinM_def)

end
