header {* The exception monad transformer as an instance of monad composition *}

theory ExnT 
imports ExnT_P
begin 



text {* Get an instance of ExceptionMonad *}

thymorph i : Main \<longrightarrow> ExnT
instantiate_theory ExceptionMonad by_thymorph i 
renames: [("ExceptionMonad.bind" \<mapsto> exn_bind),
          ("ExceptionMonad.bind'" \<mapsto> exn_bind'),
          ("ExceptionMonad.eta" \<mapsto> exn_eta),
          ("ExceptionMonad.mapM" \<mapsto> exn_mapM),
          ("ExceptionMonad.joinM" \<mapsto> exn_joinM),
          ("ExceptionMonad.bind_def" \<mapsto> exn_bind_def),
          ("ExceptionMonad.bind'_def" \<mapsto> exn_bind'_def),
          ("ExceptionMonad.eta_def" \<mapsto> exn_eta_def),
          ("ExceptionMonad.mapM_def" \<mapsto> exn_mapM_def),
          ("ExceptionMonad.joinM_def" \<mapsto> exn_joinM_def)]
without_syntax

thymorph exn_inst by_composition i' \<circ> exn_monad
renames: [("ExceptionMonad.mon_runit" \<mapsto> exn_runit),
          ("ExceptionMonad.MonadAxms_mon_lunit" \<mapsto> exn_lunit),
          ("ExceptionMonad.MonadAxms_mon_eta_inj" \<mapsto> exn_eta_inj),
          ("ExceptionMonad.mon_assoc" \<mapsto> exn_assoc)]

instantiate_theory "../MonadBase" by_thymorph exn_inst 
without_syntax

lemmas exn_bind_simps[simp] = bind.simps_1 bind.simps_2
translate_thm "MonadBase.joinM_eta" as exn_joinM_eta along exn_inst'
translate_thm "MonadBase.mapM_comp" as exn_mapM_comp along exn_inst'
translate_thm "MonadBase.mapM_eta" as exn_mapM_eta along exn_inst'

text {* Instantiate MonadComposition parameters by ExnT parameter and exn, respectively *}

thymorph p1_p2_inst : "../MonadComposition_P2" \<longrightarrow> ExnT
type_map: [("'a M1" \<mapsto> "'a M1"),
           ("'a M2" \<mapsto> "'a exn")] 
op_map: [("MonadComposition_P2.eta2" \<mapsto> "ExnT.exn_eta")]


text {* Get the actual proper instance of monad composition *}

instantiate_theory "../MonadComposition" by_thymorph p1_p2_inst

text {* Definition of swap for exceptions *}
primrec
"swap (Error e) = etaM1 (Error e)"
"swap (Just m) = (mapM1 exn_eta m)"


lemma swap_eta:
"swap \<circ> exn_eta = mapM1 exn_eta"
by(rule ext, simp add: exn_eta_def)


lemma swap_map:
"swap \<circ> exn_mapM (mapM1 f) = mapM1 (exn_mapM f) \<circ> swap"
apply(rule ext, simp add: exn_eta_def)
apply(case_tac x)
apply(simp add: exn_mapM_def)
apply(subst mapM1_eta1[THEN fun_cong, simplified])
apply(simp add: exn_mapM_def)
apply(simp add: exn_eta_def) 
apply(simp add: exn_mapM_def exn_eta_def)
apply(subst mapM1_comp[THEN fun_cong, simplified])+
apply(fold exn_eta_def)
apply(subst exn_mapM_eta)
by (rule refl)


lemma swap_map_eta:
"swap \<circ> exn_mapM etaM1 = etaM1"
apply(rule ext, simp add: exn_eta_def)
apply(case_tac x, simp_all add: exn_mapM_def exn_eta_def)
apply(subst mapM1_eta1[THEN fun_cong, simplified])
by (rule refl)

lemma prod_dorp:
"prod \<circ> exn_mapM dorp = dorp \<circ> prod"
apply(rule ext, simp add: prod_def dorp_def)
apply(case_tac x) 
apply(simp add: exn_joinM_def exn_mapM_def exn_eta_def)
apply(subst mapM1_eta1[THEN fun_cong, simplified])+
apply(subst joinM1_eta1[THEN fun_cong, simplified])
apply(simp add: exn_joinM_def)
apply(simp add: exn_joinM_def exn_mapM_def exn_eta_def)
apply(subst mapM1_comp[THEN fun_cong, simplified])+
apply(subst mapM1_joinM1_comp2[THEN fun_cong, THEN sym, simplified])
apply(subst mapM1_comp[THEN fun_cong, simplified])
apply(fold exn_eta_def)
apply(subst o_assoc[THEN sym])
apply(subst exn_joinM_eta)+
apply(subst mapM1_id)
by simp

thymorph exnT_comp by_sigmorph p1_p2_inst'
thymorph exnT_monad by_composition exnT_comp \<circ> monad_comp

text {* Get finally the syntax for the composed monad *}
instantiate_theory "../MonadBase" by_thymorph exnT_monad



text{* Raising and handling errors *}

types 'a exnT = "('a exn) M1" 

constdefs
  raise  :: "string \<Rightarrow> 'a exnT"
 "raise err \<equiv> (etaM1 (Error err))" 

constdefs
  handle :: "'a exnT \<Rightarrow> (string \<Rightarrow> 'a M1) \<Rightarrow> 'a M1"
 "handle mex f \<equiv>  (mex >>=* (\<lambda> ex. case ex of Error e => f e 
                                            | Just v  => etaM1 v))"


end



