
theory Squareroot 
imports "../../Extensions/AWE_HOL"
begin


section {* Stepwise construction of t : SplitImpl --> Squareroot *}

thymorph i : Main \<longrightarrow> Squareroot

instantiate_theory SplitSpecs by_thymorph i
type_map: [("D" \<mapsto> "nat"),
           ("R" \<mapsto> "nat")]


constdefs
 sqr :: "nat \<Rightarrow> nat"
"sqr x \<equiv> x*x"

defs
 Inv_def  : "Inv \<equiv> (\<lambda> (n, k). sqr k \<le> n)"
 Term_def : "Term \<equiv> (\<lambda> (n, k). n < sqr (Suc k))"

instantiate_theory SplitOps by_thymorph i'
(* Get the auxiliary operations. These we define as follows: *) 
defs
E_def     : "E n \<equiv> 0"

H_def     : "H \<equiv> (\<lambda> (n, k). Suc k)"

R_ord_def : "R_ord n \<equiv> {(a, b). (a \<le> n) \<and> (b \<le> n) \<and> (b < a)} "


(* A proper instantiation of SplitAxms *) 
instantiate_theory SplitAxms by_thymorph i''



subsection {* Proving the properties *}

lemma fin_np : "finite ({(a, b). a \<le> (n::nat) \<and> b \<le> n})"
by simp

lemma finR : " finite (R_ord n) "
by (simp add: R_ord_def, rule_tac B="{(a, b). a \<le> n \<and> b \<le> n}" in finite_subset,
    rule subsetI, simp add: split_def, rule fin_np)


lemma acycR : " acyclic (R_ord n)"
proof (simp add: acyclic_def, rule allI)
 fix x show "(x, x) \<notin> (R_ord n)^+"
 proof (rule ccontr, simp)
   assume a: "(x, x) \<in> (R_ord n)^+"
     have c: "\<forall> a b. (a, b) \<in> (R_ord n)^+ \<longrightarrow> b < a"
       by (intro allI, rule impI, simp add: R_ord_def, erule trancl_induct, simp_all)
     from a have "x < x" by (rule c[rule_format])
   thus "False" ..
 qed
qed


lemma Ord1 : "wf (R_ord b)"
by (rule finite_acyclic_wf, rule finR, rule acycR)



(* Now the theory morphism from SplitAxms can be defined *)
thymorph t by_sigmorph i'''


text {* Get finally the recursive definition of g ... *} 
instantiate_theory SplitImpl by_thymorph t 
renames: [("SplitImpl.f" \<mapsto> "sqrt")]

rename_thymorph t' to splitimpl_to_sqrt

text {* ... and the property of sqrt *}
translate_thm "SplitImpl.PostCond" as sqrt_prop along splitimpl_to_sqrt


end
