(*
 * The AWE-structure
 * $Id: awe.sml 1190 2009-12-07 13:51:33Z maxim $
 *)

structure AWE =
struct
 val _ = if !quick_and_dirty 
         then (Output.warning "AWE Extension Package should not be used in quick_and_dirty-mode, resetting";
               quick_and_dirty := false)
         else ();

 val ext_def_match = Unsynchronized.ref false; 
                                (* If true then the package will try to match domain definitions by 
                                   codomain theorems as well as codomain definitions. Can be useful
                                   in some few cases. *)
 val proof_search_with_simp = Unsynchronized.ref true;   
 val proof_search_depth = Unsynchronized.ref 2;
 val skip_datatypes = Unsynchronized.ref false;
 val skip_records = Unsynchronized.ref false;
 val skip_proofs = Unsynchronized.ref false;


(* *** String representations *** *)

   fun plain_repr [] = ""
     | plain_repr [(s, t)] = ("(" ^ (quote s) ^ " |-> " ^ (quote t) ^ ")")
     | plain_repr ((s, t)::m) = (("(" ^ (quote s) ^ " |-> " ^ (quote t) ^ "), ") ^ (plain_repr m)) 
   
(* *** Represent the type map as list of string pairs *** *)
   fun show_map [] = ""
     | show_map [(s, t)] = ("(" ^ (quote s) ^ " \<mapsto> " ^ (quote t) ^ ")")
     | show_map ((s, t)::m) = (("(" ^ (quote s) ^ " \<mapsto> " ^ (quote t) ^ "),\n        ") ^ (show_map m))

   fun show_map_i [] = ""
     | show_map_i [(s, t)] = ("(" ^ (quote s) ^ " |-> " ^ (quote t) ^ ")")
     | show_map_i ((s, t)::m) = (("(" ^ (quote s) ^ " |-> " ^ (quote t) ^ "),\n") 
                                  ^ (show_map_i m))

   fun show_der_map [] = ""
     | show_der_map [(s, t)] = ("Found: (" ^ (quote s) ^ " |-> " ^ (quote t) ^ ")")
     | show_der_map ((s, t)::m) = (("Found: (" ^ (quote s) ^ " |-> " ^ (quote t) ^ "),\n") ^ 
                                    (show_der_map m))

   fun show_amb_map [] = ""
     | show_amb_map [(s, t)] = ("(" ^ (quote s) ^ " |-> " ^ (commas (map quote t)) ^ ")")
     | show_amb_map ((s, t)::m) = (("(" ^ (quote s) ^ " |-> " ^ (commas (map quote t)) ^ "),\n") ^ 
                                   (show_amb_map m))

fun mk_suggestion [] = []
  | mk_suggestion ((s, is)::xs) =
    (case is of
      [] => error "Unexpected error in mk_suggestion"
    | _ => (let fun find pat [] = NONE
                  | find pat (y::ys) = (let val (q, nm) = Aux.qname_split2 y
                                        in
                                         if (pat = nm) then SOME y else find pat ys
                                        end)
                val (q, nm) = Aux.qname_split2 s
            in
             case (find nm is) of
               NONE => (s, hd is)::(mk_suggestion xs)
             | SOME y => (s, y)::(mk_suggestion xs)
            end))


(* write Isabelle/Isar keywords *)



local


val regexp_meta = member (op =) (explode ".*+?[]^$");
val regexp_quote = translate_string (fn c => if regexp_meta c then "\\\\" ^ c else c);

fun defconst name strs =
  "\n(defconst isar-keywords-" ^ name ^
  "\n  '(" ^ space_implode "\n    " (map (quote o regexp_quote) strs) ^ "))\n";


fun make_elisp_commands commands kind = defconst kind
  (commands |> map_filter (fn c => (case (OuterKeyword.command_keyword c) of
                                     NONE => NONE
                                   | SOME T => if (OuterKeyword.kind_of T = kind) then SOME c else NONE)))


fun make_elisp_syntax (keywords, commands) =
  ";;\n\
  \;; Keyword classification tables for Isabelle/Isar extended by AWE-keywords.\n\
  \;; This file was generated by " ^ Session.name () ^ " -- DO NOT EDIT!\n\
  \;;\n\
  \;;\n" ^
  defconst "major" commands ^
  defconst "minor" (filter Syntax.is_identifier keywords) ^
  implode (map (make_elisp_commands commands) OuterKeyword.kinds) ^
  "\n(provide 'isar-keywords)\n";

in

fun write_keywords s =
 (File.write (Path.explode ("isar-keywords" ^ (if s = "" then "" else "-" ^ s) ^".el"))
    (make_elisp_syntax (OuterKeyword.dest_keywords (), OuterKeyword.dest_commands ())));

end;


end;

